#include <PCF8574.h> // Biblioteca para o PCF8574
#include <Stepper.h> // Biblioteca para o Stepper Motor
#include <SoftwareSerial.h> // Biblioteca para o Bluetooth

SoftwareSerial Bluetooth(A0, A1); // Objeto Bluetooth: A0 - TX; A1 - RX

/*
 * 0 - Botão para a Luz da cozinha
 * 1 - Botão para a Luz do Quarto
 * 2 - Botão para a Luz da Sala
 * 3 - Botão para a Luz da Garagem
 * 4 - Botão para o Ar Condicionado do Quarto
 * 5 - Botão para o Ar Condicionado da Sala
 * 6 - Botão para Abrir / Fechar o portão da Garagem
 * 7 - Botão para a Luz Exterior
 */
PCF8574 pcfBotoes(0x38);
byte portasPCF[] = {0, 1, 2, 3, 4, 5, 6, 7};
byte modoPinosPCF[] = {INPUT, INPUT, INPUT, INPUT, INPUT, INPUT, INPUT, INPUT};
const byte nPortasPCF = 8;

/*
 * 6 - Ar Condicionado do Quarto
 * 7 - Ar Condicionado da Sala
 * 8 - Fim de curso do Motor do Portão 
 * 9 - Luz da cozinha
 * 10 - Luz do Quarto
 * 11 - Luz da Sala
 * 12 - Luz da Garagem
 * 13 - Luz Exterior
 */
byte portasARD[] = {6, 7, 8, 9, 10, 11, 12, 13};
byte modoPinosARD[] = {OUTPUT, OUTPUT, INPUT, OUTPUT, OUTPUT, OUTPUT, OUTPUT, OUTPUT};
bool estadoDivisao[] = {false, false, false, false, false, false, false, false}; // Tudo desligado
const byte nPortasARD = 8;
int valueBt0;
int valueBt1;
int valueBt2;
int valueBt3; 
int valueBt4;
int valueBt5;
int valueBt6;
int valueBt7;

const int nPassos = 200; // Número de passo a dar em cada "pedido"
Stepper myStepper(nPassos, 2, 3, 4, 5); // Cria o objeto myStepper
bool aberto        = false; // Variável que indica que o portão está aberto ou fechado - false -> fechado
const byte limitBt = 2;   // Posicao do array utilizada pelo botão de limite - indica que o portão chegou ao fundo
int valueLMB;             // Variável que vai guardar o valor do botão de limite

void setup() {
  Serial.begin(9600);     // Inicia o Monitor Série
  Bluetooth.begin(9600);  // Inicia o Bluetooth
  
  // pinMode das portas do PCF8574
  for(int i=0; i<nPortasPCF; i++){
    pcfBotoes.pinMode(portasPCF[i], modoPinosPCF[i]);
  }
  pcfBotoes.begin();

  // pinMode das portas do Arduíno
  for(int i=0; i<nPortasARD; i++){
    pinMode(portasARD[i], modoPinosARD[i]);
  }

  myStepper.setSpeed(150); // Velocidade do Stepper Motor
  

  /*********** Inicializar sistema ************/
  // Desligar Luzes e Ar Condicionado
  for(int i=0; i<4; i++){
    digitalWrite(portasARD[i], LOW);
  }

  // Fechar portão de garagem
  valueLMB = digitalRead(portasARD[limitBt]);
  Serial.println(valueLMB);

  if(valueLMB == 0){
    Serial.println("Portão Aberto!!! A fechar...");
    fechaPortaoAutomatico();
  }else{
    Serial.println("Portão Fechado!!! Podemos Continuar...");
  }
}

void loop() {
  // Leitura dos botões referentes à luz da casa
  leituraBotoes();

  // Leitura de dados do Bluetooth
  int valueBTH = -1;              // Inicializa a variável valueBTH a menos um (-1) 
  if(Bluetooth.available()){      // Se receber algum valor entra na condição
    valueBTH = Bluetooth.parseInt(); // Garante que o valor recebido é inteiro
    Serial.print("Valor Bluetooth: ");
    Serial.println(valueBTH);
  }

  // Ligar / Desligar Luzes da casa
  ligaDesligaLuz(valueBt0, 3);
  ligaDesligaLuz(valueBt1, 4);
  ligaDesligaLuz(valueBt2, 5);
  ligaDesligaLuz(valueBt3, 6);
  ligaDesligaLuz(valueBt7, 7); // Luz Exterior
  // Ligar / Desligar Ar Condicionado da casa
  ligaDesligaLuz(valueBt4, 0);
  ligaDesligaLuz(valueBt5, 1);
  // Abrir / Fechar Portão de garagem
  abreFechaPortaoGaragem(valueBt6);

  // Ação Pedida pelo Bluetooth
  if(valueBTH!= -1){ // Existe um pedido da App
    if(valueBTH!= 2){ // Não se trata do portão da garagem
      ligaDesligaLuz(1, valueBTH);
    } else {  // Abrir ou fechar garagem
      abreFechaPortaoGaragem(1);
    }
  }

  delay(250);
  
}
